/*
 AeroQuad v3.0.1 - February 2012
 www.AeroQuad.com
 Copyright (c) 2012 Ted Carancho.  All rights reserved.
 An Open Source Arduino based multicopter.
 
 This program is free software: you can redistribute it and/or modify 
 it under the terms of the GNU General Public License as published by 
 the Free Software Foundation, either version 3 of the License, or 
 (at your option) any later version. 
 
 This program is distributed in the hope that it will be useful, 
 but WITHOUT ANY WARRANTY; without even the implied warranty of 
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 GNU General Public License for more details. 
 
 You should have received a copy of the GNU General Public License 
 along with this program. If not, see <http://www.gnu.org/licenses/>. 
 
 Altered by Joao Diogo Falcao, Cornell University for ECE 4760. 
 For the Atmega644 Microcontroller.
 
 */

#ifndef _AQ_FLIGHT_COMMAND_READER_
#define _AQ_FLIGHT_COMMAND_READER_


/**
 * readPilotCommands
 * 
 * This function is responsible to read receiver
 * and process command from the users
 */
void readPilotCommands() {
  
  readReceiver(); 
  if (receiverCommand[THROTTLE] < MINCHECK) {
    zeroIntegralError();

    // Disarm motors (left stick lower left corner)
    if (receiverCommand[ZAXIS] < MINCHECK && motorArmed == ON) {
      commandAllMotors(MINCOMMAND);
      PORTD |= (1<<PIN2);//digitalWrite(LED_Red,LOW);  WARNING
      motorArmed = OFF;
            
    }    
    
    // Zero Gyro and Accel sensors (left stick lower left, right stick lower right corner)
    if ((receiverCommand[ZAXIS] < MINCHECK) && (receiverCommand[XAXIS] > MAXCHECK) && (receiverCommand[YAXIS] < MINCHECK)) {
      calibrateGyro();
      computeAccelBias();
      storeSensorsZeroToEEPROM();
      calibrateKinematics();
      zeroIntegralError();
      pulseMotors(3);
    }   
    
    // Arm motors (left stick lower right corner)
    if (receiverCommand[ZAXIS] > MAXCHECK && motorArmed == OFF && safetyCheck == ON) {
      zeroIntegralError();
      commandAllMotors(MINTHROTTLE);
      PORTD &= ~(1<<PIN2); //digitalWrite(LED_Red,HIGH);   WARNING
      motorArmed = ON;
        
    }

    // Prevents accidental arming of motor output if no transmitter command received
    if (receiverCommand[ZAXIS] > MINCHECK) {
      safetyCheck = ON; 
    }

    // If motors armed, and user starts to arm/disarm motors (yaw stick hasn't passed MAXCHECK or MINCHECK yet)
    // This prevents unwanted spinup of motors
    if (motorArmed == ON) {
      commandAllMotors(MINTHROTTLE);
    }
  }
  
  #ifdef RateModeOnly
    flightMode = RATE_FLIGHT_MODE;
  #else
    // Check Mode switch for Acro or Stable
    if (receiverCommand[MODE] > 1500) {
      flightMode = ATTITUDE_FLIGHT_MODE;
   }
    else {
      flightMode = RATE_FLIGHT_MODE;
    }
  #endif  
  
}

#endif // _AQ_FLIGHT_COMMAND_READER_


